% Final Project: Stalagmite simulation in 3D. 
% The goal of this projet is to use 3D DLA to 
% simulate stalagmite growth in a cave. 

%clear; clf;

% initial conditions
NX = 100;
NY = 100;
ceilingHeight = 500;
xRange = [0, NX];
yRange = [0, NY];
zRange = [0, ceilingHeight];
growSurf = zeros(NX, NY); % create the initial surface plot

% speed up the simulation
skip = 10;
fcount = 0;
f = 0;
saveToFile = 0;

nParticles = 2000; % amount of particles to rain down
bumpHeight = floor(8);
bumpRadius = floor(6); 

global xRange yRange zRange

% initial draw
fancyPlot(growSurf);


for i=1:nParticles
    
    x = NX/2; % start at a random X
    y = NY/2; % start at a random Y
    z = ceilingHeight;
    
    tallest = max(growSurf);
    
    while tallest <= ceilingHeight + ceilingHeight/3
        xOld = x; %store current x and y values
        yOld = y;
        zOld = z;
        
        % Arbitrarily select direction to turn to
		r = rand();
        
        %%% Determine the direction of motion using the random number 'r'
        %%% Update 'x' and 'y' with the following probabilities
        %%% 60% stay in place
        %%% 10% go LEFT
        %%% 10% go RIGHT
        %%% 10% go FRONT
        %%% 10% go BACK
        if r < .6
            x = x;
        elseif r < .7
            x = boundaries(xOld - 1,NX); 
        elseif r < .8
            x = boundaries(xOld + 1,NX);
        elseif r < .9
            y = boundaries(yOld - 1,NY);
        else
            y = boundaries(yOld + 1,NY);
        end
        
        z = z-1; % always descend the particle by 1
        
        xR = growSurf(boundaries(x+1,NX),y);
        xL = growSurf(boundaries(x-1,NX),y);
        yR = growSurf(x,boundaries(y+1,NY));
        yL = growSurf(x,boundaries(y-1,NY));
        zD = growSurf(x,y);
        
        
        p = rand();
        %stickingProbability = z/ceilingHeight + 0.01;
        stickingProbability = 0.08;
        if z < tallest
            
            fcount = fcount + 1;
            if p <= stickingProbability
                if (xR > 0 || xL > 0 || yR > 0 || yL > 0 || zD > 0 || z <= 0)

                    % add a hill
                    growSurf = add_hill(growSurf, NX, NY, x, y, bumpRadius, bumpHeight);
                    if mod(fcount, skip) == 0
                        fancyPlot(growSurf);
                        if saveToFile == 1
                            saveas(gcf,['movie_',sprintf('%03d',f),'.png']);
                            f = f+1;
                        end
                    end
                    break;
                end
            elseif z <= 0

                % add a hill
                growSurf = add_hill(growSurf, NX, NY, x, y, bumpRadius, bumpHeight);
                if mod(fcount, skip) == 0
                    fancyPlot(growSurf);
                    if saveToFile == 1
                        saveas(gcf,['movie_',sprintf('%03d',f),'.png']);
                        f = f+1;
                    end
                end
                break;
                
            end
        end
        
        %sprintf('particle %d\nx=%d y=%d z=%d',i,x,y,z)
    end
    
    
end